function model = team_training_code(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%
% input_directory= fullfile('C:\Users\Xin\Desktop\the-circor-digiscope-phonocardiogram-dataset-1.0.3\the-circor-digiscope-phonocardiogram-dataset-1.0.3\training_data/')
% input_directory= fullfile([D_Path, '\Heart Sound\physionet 2022\official phase\the-circor-digiscope-phonocardiogram-dataset-1.0.3\the-circor-digiscope-phonocardiogram-dataset-1.0.3\training_data/'])
% input_directory2= fullfile('training_data_2016/')
% output_directory= fullfile('training_data/')
%% LOADING MODEL
urlwrite('https://dl.dropboxusercontent.com/s/p5gh8lz6ziq10ep/model.mat?dl=0', 'model.mat')
load model

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Seg_dur=30; % 9 seconds as a unit
Fs =4000;
signalLength = Seg_dur*Fs;

if not(isfolder('data_murmur'))
    mkdir('data_murmur');
end


if not(isfolder('data_outcome'))
    mkdir('data_outcome');
end

%%
% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes_murmur={};
classes_outcome={};
for j=1:num_patient_files

    current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
    classes_murmur=unique([classes_murmur current_class_murmur]);

    current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
    classes_outcome=unique([classes_outcome current_class_outcome]);

end

classes_murmur=sort(classes_murmur);
num_classes_murmur=length(classes_murmur);

classes_outcome=sort(classes_outcome);
num_classes_outcome=length(classes_outcome);

%


for i=1:length(classes_murmur)
    if not(isfolder(fullfile('data_murmur/', char(classes_murmur{1,i}))))
        mkdir(fullfile('data_murmur', char(classes_murmur{1,i})));
    end
end


for i=1:length(classes_outcome)
    if not(isfolder(fullfile('data_outcome/', char(classes_outcome{1,i}))))
        mkdir(fullfile('data_outcome', char(classes_outcome{1,i})));
    end
end


% Extracting features and labels
disp('Extracting features and labels...')

features=[];
labels_murmur=categorical;
labels_outcome=categorical;

for j=1:num_patient_files

    fprintf('%d/%d \n',j,num_patient_files)

    current_header=get_header(fullfile(input_directory,patient_files{j}));
    current_recordings=load_recordings(input_directory,current_header);

    %     current_features=get_features(current_header, current_recordings);
    %     features(j,:) = current_features(:);

    labels_murmur(j)=get_class_murmur(fullfile(input_directory,patient_files{j}));
    labels_outcome(j)=get_class_outcome(fullfile(input_directory,patient_files{j}));
    %
    temp=str2num(current_header{1});

    Fs_current =temp(end);% added

    current_recordings=load_recordings(input_directory,current_header);

    temp=str2num(current_header{1, 1});
    recording_files=get_recording_files(current_header);
    %

    % processing from here
    for i=1:length(current_recordings)

        SIG=current_recordings{1, i};
        if Fs_current==2000;
            SIG=resample(SIG, 2,1);
        end
        if length(SIG)>=signalLength

            sig=SIG(1:signalLength);
        else
            sig=[SIG;zeros(signalLength-length(SIG),1)]; % padding if shorter than predefiend
        end

        for loop =1:4
            switch loop
                case 1
                    factor =0;
                case 2
                    factor =0.5;
                case 3
                    factor =1;
                case 4
                    factor =1.5;
            end
            for loop2= 1:2
                switch loop2
                    case 1
                        sig=sig+randn(length(sig),1)*std(sig)*factor;
                    case 2
                        % apply time shift
                        if length(SIG)>=signalLength

                            sig=SIG(end-signalLength+1:end);
                            sig=sig+randn(length(sig),1)*std(sig)*factor;
                        else
                            sig(1:2*Fs_current)=[];
                            sig=[sig;zeros(signalLength-length(sig),1)];
                        end
                end


                % [S,F,T] = stft(sig,Fs,'FrequencyRange','onesided');
                %         [S,F,T] = melSpectrogram(sig,Fs,'NumBands',13);
                [S,F,T] = melSpectrogram(sig,Fs, ...
                    'NumBands',64, ...
                    'FrequencyRange',[25,2000]);
                smag = mag2db(abs(S)); % Convert to dB
                % caxisLims = max(smag(:)) + [-100 0]; % Color axis limits
                %

                %         smag(smag<(max(smag(:))-100))=min(smag(:));
                test=smag;
                %         test=[smag(:, 1:224);smag(:, 225:224*2);smag(:, 449:224*3);smag(:, 673:896);];
                test(test==-inf)=nan;
                im = ind2rgb(im2uint8(rescale(test)),jet);


                imageRoot = fullfile(pwd,'data_murmur/');
                % im = ind2rgb(im2uint8(rescale(wt)),jet(128));
                imgLoc = fullfile(imageRoot,char(labels_murmur(j)));
                imFileName = strcat([recording_files{1, i}(1:end-4),num2str(loop),num2str(loop2)],'.png');
                imwrite(imresize(im,[227 227]),fullfile(imgLoc,imFileName),'png');


                imageRoot = fullfile(pwd,'data_outcome/');
                % im = ind2rgb(im2uint8(rescale(wt)),jet(128));
                imgLoc = fullfile(imageRoot,char(labels_outcome(j)));
                imFileName = strcat([recording_files{1, i}(1:end-4),num2str(loop),num2str(loop2)],'.png');
                imwrite(imresize(im,[227 227]),fullfile(imgLoc,imFileName),'png');
            end
        end
        % end
    end

end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MURMUR MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%
rootFolder = pwd;
folderName = 'data_murmur';

rng('default');
allImages = imageDatastore(strcat(rootFolder,filesep,folderName), 'IncludeSubfolders', true,...
    'LabelSource', 'foldernames');
% % over sampliing
% labelCount = countEachLabel(allImages);
% labels=allImages.Labels;
% [G,classes1] = findgroups(labels);
% numObservations = splitapply(@numel,labels,G);
% desiredNumObservationsPerClass = max(numObservations);
% 
% files = splitapply(@(x){randReplicateFiles(x,desiredNumObservationsPerClass)},allImages.Files,G);
% files = vertcat(files{:});
% labels=[];
% try
%     info=strfind(files,'\');
% for i=1:numel(files)
%     idx=info{i};
%     dirName=files{i};
%     targetStr=dirName(idx(end-1)+1:idx(end)-1);
%     targetStr2=cellstr(targetStr);
%     labels=[labels;categorical(targetStr2)];
% end
% catch
%      info=strfind(files,'/');
% for i=1:numel(files)
%     idx=info{i};
%     dirName=files{i};
%     targetStr=dirName(idx(end-1)+1:idx(end)-1);
%     targetStr2=cellstr(targetStr);
%     labels=[labels;categorical(targetStr2)];
% end
% end
% 
% allImages.Files = files;
% allImages.Labels=labels;
% labelCount_oversampled = countEachLabel(allImages);
%
% load alexnet;
layers = model_murmur.Layers


ilr = 0.0001;
mxEpochs = 20;
mbSize = 64;
opts = trainingOptions('sgdm', 'InitialLearnRate', ilr, ...
    'MaxEpochs',mxEpochs , 'MiniBatchSize',mbSize, ...
     'Shuffle','once');

allImages.ReadFcn = @readFunctionTrain;
[model_murmur, info] = trainNetwork(allImages, layers, opts); %myNe



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% OUTCOME MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%
rootFolder = pwd;
folderName = 'data_outcome';

rng('default');
allImages = imageDatastore(strcat(rootFolder,filesep,folderName), 'IncludeSubfolders', true,...
    'LabelSource', 'foldernames');
% over sampliing
% labelCount = countEachLabel(allImages);
% % histogram(imdsTrain.Labels)
% labels=allImages.Labels;
% [G,classes1] = findgroups(labels);
% numObservations = splitapply(@numel,labels,G);
% % desiredNumObservationsPerClass is the maximum number of sample among all classes.
% 
% desiredNumObservationsPerClass = max(numObservations);
% 
% files = splitapply(@(x){randReplicateFiles(x,desiredNumObservationsPerClass)},allImages.Files,G);
% files = vertcat(files{:});
% labels=[];
% 
% try
%     info=strfind(files,'\');
% for i=1:numel(files)
%     idx=info{i};
%     dirName=files{i};
%     targetStr=dirName(idx(end-1)+1:idx(end)-1);
%     targetStr2=cellstr(targetStr);
%     labels=[labels;categorical(targetStr2)];
% end
% catch
%      info=strfind(files,'/');
% for i=1:numel(files)
%     idx=info{i};
%     dirName=files{i};
%     targetStr=dirName(idx(end-1)+1:idx(end)-1);
%     targetStr2=cellstr(targetStr);
%     labels=[labels;categorical(targetStr2)];
% end
% end
% 
% allImages.Files = files;
% allImages.Labels=labels;
% labelCount_oversampled = countEachLabel(allImages);

layers = model_outcome.Layers


ilr = 0.0001;
mxEpochs = 20;
mbSize = 64;
opts = trainingOptions('sgdm', 'InitialLearnRate', ilr, ...
    'MaxEpochs',mxEpochs , 'MiniBatchSize',mbSize, ...
    'Shuffle','once');

allImages.ReadFcn = @readFunctionTrain;

[model_outcome, info] = trainNetwork(allImages, layers, opts); %myNe

%% SAVE MODEL


% save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory);
% output_directory='C:\Users\Xin\Desktop'
save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory);
disp('Done.')

end

function save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur','classes_murmur','model_outcome','classes_outcome','-v7.3');

disp('Done.')
end

function class=get_class_murmur(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function class=get_class_outcome(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Outcome'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)

    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));

end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1

    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};

end

end



function saveTimeFrequencyRep(wt,ii,labels)

imageRoot = fullfile(pwd,'data');
im = ind2rgb(im2uint8(rescale(wt)),jet(128));
imgLoc = fullfile(imageRoot,char(labels(ii)));
imFileName = strcat(char(labels(ii)),'_',num2str(ii),'.png');
imwrite(imresize(im,[224 224]),fullfile(imgLoc,imFileName),'png');

end


function I2 = readFunctionTrain(filename)
% resize the ekg data
I = imread(filename);

I2 = imresize(I, [227 227]);
end

function files = randReplicateFiles(files,numDesired)
n = numel(files);
ind = randi(n,numDesired,1);
files = files(ind);
end
